#include "ai.h"


Ai::Ai(QObject *_parent,  Deckmodel *_pfield, Deckmodel *_ofield, Deckmodel *_ohand, Deckmodel *_olib, Gamestate *_gamestate): pfield(_pfield), ofield(_ofield), ohand(_ohand), olib(_olib), gamestate(_gamestate), turnCounter(0)
{
	gamestate->shuffleEnemyLib();
	for(int i = 0; i < 7; i++)
	{
		if(olib->cards.size() > 0)
		{
			gamestate->moveCardPosition(Card::POS_OLIB, Card::POS_OHAND,olib->cards.first().id);
		}
	}

	while(noHandCardsAreLands())
	{
		redrawSevenCards();
	}
}

int Ai::countOwnLandsPlayfield()
{
	const QVector<Card>& cards = this->ofield->cards;
	int counter = 0;

	for(int i = 0; i < cards.size(); i++)
	{
		if(cards[i].type == Card::TYPE_LAND)
		{
			counter++;
		}
		else if(cards[i].description.contains("↺: Add"))
		{
			counter++;
		}
	}
	return counter;
}

int Ai::countOwnLandsHand()
{
	const QVector<Card>& cards = this->ohand->cards;
	int counter = 0;

	for(int i = 0; i < cards.size(); i++)
	{
		if(cards[i].type == Card::TYPE_LAND)
		{
			counter++;
		}
	}
	return counter;
}

int Ai::countEnemyCreatures()
{
	const QVector<Card>& cards = this->pfield->cards;
	int counter = 0;

	for(int i = 0; i < cards.size(); i++)
	{
		if(cards[i].type == Card::TYPE_CREATURE)
		{
			counter++;
		}
	}
	return counter;
}

int Ai::countOwnCreatures()
{
	const QVector<Card>& cards = this->ofield->cards;
	int counter = 0;

	for(int i = 0; i < cards.size(); i++)
	{
		if(cards[i].type == Card::TYPE_CREATURE)
		{
			counter++;
		}
	}
	return counter;
}

int Ai::getFirstLandId()
{
	const QVector<Card>& cards = this->ohand->cards;

	for(int i = 0; i < cards.size(); i++)
	{
		if(cards[i].type == Card::TYPE_LAND)
		{
			return cards[i].id;
		}
	}
	return -2;
}

int Ai::getIdOfPlayableCard(int maxcost)
{
	const QVector<Card>& cards = this->ohand->cards;

	for(int i = 0; i < cards.size(); i++)
	{
		if(cards[i].cost <= maxcost && cards[i].type != Card::TYPE_LAND)
		{
			return cards[i].id;
		}
	}
	return -2;
}

const Card* Ai::getPlayableCard(int maxcost)
{
	const QVector<Card>& cards = this->ohand->cards;

	for(int i = 0; i < cards.size(); i++)
	{
		if(cards[i].cost <= maxcost && cards[i].type != Card::TYPE_LAND)
		{
			return &cards[i];
		}
	}
	return NULL;
}

Card *Ai::getPlayableCard(int maxcost, const QVector<const Card *> &skippedCards)
{
	QVector<Card>& cards = this->ohand->cards;

	for(int i = 0; i < cards.size(); i++)
	{
		if(cards[i].cost <= maxcost && cards[i].type != Card::TYPE_LAND && !skippedCards.contains(&cards[i]))
		{
			return &cards[i];
		}
	}
	return NULL;
}

Card *Ai::getBestPlayableCard(int maxcost, const QVector<const Card *> &skippedCards)
{
	QVector<Card>& cards = this->ohand->cards;
	const int numberOfLands = this->countOwnLandsPlayfield();
	const int numberOfCreatures = this->countOwnCreatures();
	int lowest_useless_value = 1000000;
	Card* bestCard = NULL;

	for(int i = 0; i < cards.size(); i++)
	{
		if(cards[i].cost <= maxcost && cards[i].type != Card::TYPE_LAND && !skippedCards.contains(&cards[i]))
		{
			int metric = calculateManaLandDistance(cards[i],numberOfLands) + calculateLowCreatureCountPenality(cards[i],numberOfCreatures);

			if(numberOfLands <= MANA_ARTIFACTS_TRESHOLD && isManaSource(cards[i]))
			{
				metric = -1;
			}

			if(metric < lowest_useless_value)
			{
				lowest_useless_value = metric;
				bestCard = &cards[i];
			}
		}
	}
	if(bestCard != NULL)
	{
		qDebug() << "Best card: " << bestCard->name;
	}
	return bestCard;
}

void Ai::followCardInstrunction(const Card* card)
{
	followCardInstrunctionDraw(card);
	followCardInstrunctionDiscard(card);
}

void Ai::followCardInstrunctionDraw(const Card *card)
{
	const QString description (getDrawSentence(card));

	if(!description.contains("draw"))
	{
		return;
	}

	const int cards_to_draw = getDrawNumber(description);
	for(int i = 0; i < cards_to_draw; i++)
	{
		drawCard();
	}
}

QString Ai::getDrawSentence(const Card *card)
{
	const QString& desc(card->description.toLower());
	QStringList splitted;

	if (!desc.contains("draw"))
	{
		return QString("");
	}

	if(desc.contains("."))
	{
		splitted = desc.split(".");

		if(desc.contains(","))
		{
			for (int i = 0; i < splitted.size(); i++)
			{
				if(splitted.at(i).contains(","))
				{
					QStringList subsplit = splitted.at(i).split(",");
					for (int x = 0; x < subsplit.size(); x++)
					{
						const QString subsentence = subsplit.at(x);
						if(subsentence.contains("draw"))
						{
							return subsentence;
						}
					}
				}
				else
				{
					if(splitted.at(i).contains("draw"))
					{
						return splitted.at(i);
					}
				}
			}
		}
		else
		{
			for(int i = 0; i < splitted.size(); i++)
			{
				if(splitted.at(i).contains("draw"))
				{
					return splitted.at(i);
				}
			}
		}
	}
	else if(desc.contains(","))
	{
		for(int i = 0; i < splitted.size(); i++)
		{
			if(splitted.at(i).contains("draw"))
			{
				return splitted.at(i);
			}
		}
	}
	return QString("");
}

int Ai::getDiscardNumber(const QString &sentence)
{
	return sentence.contains("a card") ? 1 :
	(sentence.contains("two") ? 2 : (sentence.contains("three") ? 3 : (sentence.contains("four") ? 4 : (sentence.contains("five") ? 5 :  0))));
}

QString Ai::getDiscardSentence(const Card *card)
{
	const QString& desc(card->description.toLower());
	QStringList splitted;

	if (!desc.contains("discard") || desc.contains("madness") || desc.contains("Jump-start"))
	{
		return QString("");
	}

	if(desc.contains("."))
	{
		splitted = desc.split(".");

		if(desc.contains(","))
		{
			for (int i = 0; i < splitted.size(); i++)
			{
				if(splitted.at(i).contains(","))
				{
					QStringList subsplit = splitted.at(i).split(",");
					for (int x = 0; x < subsplit.size(); x++)
					{
						const QString subsentence = subsplit.at(x);
						if(subsentence.contains("discard"))
						{
							return subsentence;
						}
					}
				}
			}
		}
		else
		{
			for(int i = 0; i < splitted.size(); i++)
			{
				if(splitted.at(i).contains("discard"))
				{
					return splitted.at(i);
				}
			}
		}
	}
	else if(desc.contains(","))
	{
		for(int i = 0; i < splitted.size(); i++)
		{
			if(splitted.at(i).contains("discard"))
			{
				return splitted.at(i);
			}
		}
	}
	return QString("");
}

void Ai::tapLands(int amount)
{
	if(amount < 1)
	{
		return;
	}

	QVector<Card>& cards = gamestate->opponent_field;

	for(int i = 0; i < cards.size(); i++)
	{
		if(cards[i].type == Card::TYPE_LAND || isManaSource(cards[i]))
		{
			if(amount > 0)
			{
				cards[i].tapped = 1;
				amount--;
				if(amount <= 0)
				{
					break;
				}
			}
		}
	}
}

void Ai::untapEverything()
{
	QVector<Card>& cards = gamestate->opponent_field;

	for(int i = 0; i < cards.size(); i++)
	{
		cards[i].tapped = 0;
	}
}

bool Ai::isCardUseful(const Card *card)
{

	const QString desc = card->description.toLower();


	qDebug() << "oldcreatures.size()" << oldcreatures.size();

	if (
			( ((desc.contains("target") && desc.contains("creature gets") ) && desc.contains("+") )	||	(desc.contains("target") && desc.contains("creature gains")) )
			&& oldcreatures.size() <= 0 && card->type == Card::TYPE_SPELL)
	{
		return false;
	}

	else if (
				(
				(desc.contains("each creature") && desc.contains("damage")) ||
				(desc.contains("damage") && desc.contains("target creature"))
				)
				&& countEnemyCreatures() <= 0 && card->type == Card::TYPE_SPELL)
	{
		return false;
	}

	return true;
}

void Ai::discardDownToSevenCards()
{
	while(this->gamestate->opponent_hand.size() > NUMBER_OF_HANDCARDS)
	{
		discardUselessCard();
	}
}

bool Ai::isManaSource(const Card& card)
{
	return card.description.contains("↺: Add");
}

bool Ai::isInOldCreatures(Card *card) const
{
	if(card->type == Card::TYPE_CREATURE)
	{
		return oldcreatures.contains(card->id);
	}
	return false;
}

bool Ai::noHandCardsAreLands()
{
	QVector<Card>& cards = gamestate->opponent_hand;

	for(int i = 0; i < cards.size();i++)
	{
		if(cards[i].type == Card::TYPE_LAND)
		{
			return false;
		}
	}
	return true;
}

void Ai::redrawSevenCards()
{
	while(gamestate->opponent_hand.size() > 0)
	{
		gamestate->moveCardPosition(Card::POS_OHAND, Card::POS_OLIB,ohand->cards.first().id);
	}

	gamestate->shuffleEnemyLib();
	while(gamestate->opponent_hand.size() < 7)
	{
		gamestate->moveCardPosition(Card::POS_OLIB, Card::POS_OHAND,olib->cards.first().id);
	}

}

void Ai::addCreaturesToOldCreatures()
{
	oldcreatures.clear();
	QVector<Card>& cards = gamestate->opponent_field;

	for(int i = 0; i < cards.size();i++)
	{
		if(cards[i].type == Card::TYPE_CREATURE)
		{
			oldcreatures.push_back(cards[i].id);
		}
	}
}


void Ai::sanatizeOldCreatures()
{
	QVector<Card>& cards = gamestate->opponent_field;
	QVector<int> saniCreaturesId;
	QVector<int> fieldCreaturesId;

	for(int i = 0; i < cards.size();i++)
	{
		if(cards[i].type == Card::TYPE_CREATURE)
		{
			fieldCreaturesId.push_back(cards[i].id);
		}
	}


	for(int i = 0; i < oldcreatures.size();i++)
	{
		if(fieldCreaturesId.contains(oldcreatures[i]))
		{
			saniCreaturesId.append(oldcreatures[i]);
		}
	}
	oldcreatures = saniCreaturesId;

}


void Ai::followCardInstrunctionDiscard(const Card *card)
{
	const QString description = getDiscardSentence(card);
	//const QString description = card->description.toLower();

	if(!description.contains("discard") || description.contains("madness") || description.contains("↺, discard"))
	{
		return;
	}

	int number = getDiscardNumber(description);

	if(description.contains("random"))
	{
		for(int i = 0; i < number; i++)
		{
			discardRandomCard();
		}
	}
	else
	{
		for(int i = 0; i < number; i++)
		{
			discardUselessCard();
		}
	}


}

int Ai::getDrawNumber(const QString &sentence)
{
	return sentence.contains("draw a card") ? 1 :
	(sentence.contains("two") ? 2 : (sentence.contains("three") ? 3 : (sentence.contains("four") ? 4 : (sentence.contains("five") ? 5 :  0))));

}


void Ai::playRandomCard()
{
	if(ohand->cards.size() > 0)
	{
		gamestate->moveCardPosition(Card::POS_OHAND, Card::POS_OFIELD,ohand->cards[rand()%ohand->cards.size()].id);
	}
}

void Ai::drawCard()
{
	if(olib->cards.size() > 0)
	{
		gamestate->moveCardPosition(Card::POS_OLIB, Card::POS_OHAND,olib->cards.first().id);
	}
}

void Ai::discardRandomCard()
{
	if(ohand->cards.size() > 0)
	{
		const int random = rand() % ohand->cards.size();
		gamestate->moveCardPosition(Card::POS_OHAND, Card::POS_OGRAVE,ohand->cards[random].id);
	}
}

void Ai::discardUselessCard()
{
	if(ohand->cards.size() <= 0)
	{
		return;
	}

	int uselessness = 0;
	const int numberOfLands = countOwnLandsPlayfield();
	QVector<Card>& cards = gamestate->opponent_hand;
	int index = 0;

	for (int i = 0; i < cards.size(); i++)
	{
		int currentUselessness = calculateLandNegativeValue(cards[i], numberOfLands) + calculateManaLandDistance(cards[i], numberOfLands);

		if(numberOfLands < MANA_ARTIFACTS_TRESHOLD && isManaSource(cards[i]))
		{
			currentUselessness = 0;
		}

		if(currentUselessness > uselessness)
		{
			uselessness = currentUselessness;
			index = i;
		}
	}

	gamestate->moveCardPosition(Card::POS_OHAND, Card::POS_OGRAVE,ohand->cards[index].id);
}

void Ai::AiActionRandom()
{
	drawCard();
	playRandomCard();

	if(turnCounter > 3)
	{
		playRandomCard();
	}

	turnCounter++;
}

void Ai::AiActionPlayLandAndLegalCard()
{
	drawCard();
	if(countOwnLandsHand() > 0)
	{
		const int id = getFirstLandId();
		if(id >= 0)
		{
			gamestate->moveCardPosition(Card::POS_OHAND, Card::POS_OFIELD,id);
		}
	}

	const int cardid = getIdOfPlayableCard(countOwnLandsPlayfield());
	if(cardid >= 0)
	{
		gamestate->moveCardPosition(Card::POS_OHAND, Card::POS_OFIELD,cardid);
	}

}

void Ai::AiActionPlayLandLoopLegalCard()
{
	drawCard();
	const int number_of_lands = countOwnLandsHand();
	if(number_of_lands > 0)
	{
		const int id = getFirstLandId();
		if(id >= 0)
		{
			gamestate->moveCardPosition(Card::POS_OHAND, Card::POS_OFIELD,id);
		}
	}
	int mana = countOwnLandsPlayfield();
	while(mana > 0)
	{
		const Card* card = getPlayableCard(mana);
		if(card != NULL && mana > 0)
		{
			mana = mana - card->cost;
			gamestate->moveCardPosition(Card::POS_OHAND, Card::POS_OFIELD,card->id);

		}
		else
		{
			break;
		}
	}

	ofield->beginResetModel();
	this->ofield->sortDeck();
	ofield->endResetModel();


}

void Ai::AiActionPlayLandLoopUsefulCard()
{
	removeNonPermanentsFromPlayingField();
	sanatizeOldCreatures();
	untapEverything();
	drawCard();
	const int number_of_lands = countOwnLandsHand();
	if(number_of_lands > 0)
	{
		const int id = getFirstLandId();
		if(id >= 0)
		{
			gamestate->moveCardPosition(Card::POS_OHAND, Card::POS_OFIELD,id);
		}
	}
	int mana = countOwnLandsPlayfield();
	const int start_mana = mana;


	QVector<const Card*> skippedCards;
	int lowest_useless_value = 10000000;
	int best_card_index = 0;

	int extra_mana_counter = 0;

	while(mana > 0)
	{
		//const Card* card = getPlayableCard(mana,skippedCards);
		Card* card = getBestPlayableCard(mana,skippedCards);

		if(card != NULL && mana > 0)
		{
			if(card->cost > mana)
			{
				qDebug() << "Card cost > mana, this should not happen." << card->name << " " << card->cost << " Mana: " << mana;
				break;
			}


			if (!isCardUseful(card))
			{
				skippedCards.push_back(card);
				continue;
			}

			mana = mana - card->cost;

			Card temp = *card;
			if(isManaSource(temp))
			{
				mana++;
				extra_mana_counter++;
				card->tapped=1;
			}

			gamestate->moveCardPosition(Card::POS_OHAND, Card::POS_OFIELD,card->id);
			followCardInstrunction(&temp);



		}
		else if (skippedCards.size() >= gamestate->opponent_hand.size())
		{
			break;
		}
		else
		{
			break;
		}
	}
	tapLands(start_mana+extra_mana_counter-mana);
	discardDownToSevenCards();
	addCreaturesToOldCreatures();

	ofield->beginResetModel();
	this->ofield->sortDeck();
	//gamestate->setChatText("===END AI TURN===\n");
	ofield->endResetModel();
}

void Ai::removeNonPermanentsFromPlayingField()
{
	QVector<Card>& cards = gamestate->opponent_field;
	for(int i = 0; i < cards.size(); i++)
	{
		if(cards[i].type == Card::TYPE_SPELL)
		{
			gamestate->moveCardPosition(Card::POS_OFIELD, Card::POS_OGRAVE,cards[i].id);
		}
	}
}
