#ifndef AI_H
#define AI_H

#include <QObject>

#include "gamestate.h"
#include "deckmodel.h"

class Ai : public QObject
{
	Q_OBJECT

	Deckmodel* pfield;
	Deckmodel* ofield;
	Deckmodel* ohand;
	Deckmodel* olib;
	Gamestate* gamestate;
	int turnCounter;
	QVector<int> oldcreatures;


public:
	Ai(QObject *_parent, Deckmodel* _pfield, Deckmodel* _ofield, Deckmodel* _ohand, Deckmodel* _olib, Gamestate* _gamestate);
	int countOwnLandsPlayfield();
	int countOwnLandsHand();
	int countEnemyCreatures();
	int countOwnCreatures();
	int getFirstLandId();
	int getIdOfPlayableCard(int maxcost);
	const Card* getPlayableCard(int maxcost);
	Card* getPlayableCard(int maxcost, const QVector<const Card*>& skippedCards);
	Card* getBestPlayableCard(int maxcost, const QVector<const Card*>& skippedCards);
	void followCardInstrunction(const Card* card);
	void followCardInstrunctionDraw(const Card* card);
	void followCardInstrunctionDiscard(const Card* card);
	int getDrawNumber(const QString& sentence);
	QString getDrawSentence(const Card* card);

	int getDiscardNumber(const QString& sentence);
	QString getDiscardSentence(const Card* card);
	void tapLands(int amount);
	void untapEverything();
	bool isCardUseful(const Card* card);
	void discardDownToSevenCards();
	bool isManaSource(const Card& card);


private:
	const int NUMBER_OF_HANDCARDS = 7;
	const int FACTOR_TO_MANY_LANDS=5;
	const int MANA_ARTIFACTS_TRESHOLD=4; //up to four mana, play mana artifacts with priority
	const int CREATURE_COUNT_TRESHOLD=3; //up to four mana, play mana artifacts with priority
	const int CREATURE_COUNT_PENALITY=3; //up to four mana, play mana artifacts with priority

	inline int calculateManaLandDistance(const Card& card, const int numberOfLands) const
	{
		return std::abs(numberOfLands - card.cost);
	}
	inline int calculateLandNegativeValue(const Card &card, const int numberOfLands) const
	{
		return (numberOfLands > 5 ? 1 : 0) * (card.type == Card::TYPE_LAND ? 1 : 0) * FACTOR_TO_MANY_LANDS;
	}
	inline int calculateLowCreatureCountPenality(const Card &card, const int numberOfCreatures)
	{
		return (numberOfCreatures < 3 ? 1 : 0) * (card.type == Card::TYPE_CREATURE ? 0 : 1) * CREATURE_COUNT_PENALITY;
	}

	void addCreaturesToOldCreatures();
	void sanatizeOldCreatures();
	bool isInOldCreatures(Card* card) const;
	bool noHandCardsAreLands();
	void redrawSevenCards();


signals:

public slots:
	Q_INVOKABLE void playRandomCard();
	Q_INVOKABLE void drawCard();
	Q_INVOKABLE void discardRandomCard();
	Q_INVOKABLE void discardUselessCard();
	Q_INVOKABLE void AiActionRandom();
	Q_INVOKABLE void AiActionPlayLandAndLegalCard();
	Q_INVOKABLE void AiActionPlayLandLoopLegalCard();
	Q_INVOKABLE void AiActionPlayLandLoopUsefulCard();
	Q_INVOKABLE void removeNonPermanentsFromPlayingField();

};

#endif // AI_H
