#include "card.h"

Card::Card(const QString &_name, const QString &_edition, const QString &_description, const QString &_type, const QString& _cost, const int _pos) :
	name(_name), edition(_edition), description(_description), position(_pos), id(card_counter), type(TypeStringToEnum(_type)), commander(TypeToCommander(_type)), cost(costToInt(_cost))
{
    connect(&Card::webapi, SIGNAL(downloadDone(QString,QString,QString)), this, SLOT(setUrl(QString,QString,QString)));
    webapi.getUrl(_name);
    card_counter++;
}

Card::Card(const QString &_name, const int _pos) : name(_name), edition("none"), description("none"), position(_pos), id(card_counter), type(TYPE_OTHER), commander(0), cost(0)
{
    connect(&Card::webapi, SIGNAL(downloadDone(QString,QString,QString)), this, SLOT(setUrl(QString,QString,QString)));
    webapi.getUrl(_name);
    card_counter++;

}

Card::Card(const Card & _other):  QObject (Q_NULLPTR), name(_other.name), edition(_other.edition), description(_other.description), position(_other.position), is_selected(_other.is_selected), id(_other.id), tapped(_other.tapped), type(_other.type), commander(_other.commander), cost(_other.cost)
{
    connect(&Card::webapi, SIGNAL(downloadDone(QString,QString,QString)), this, SLOT(setUrl(QString,QString,QString)));
    webapi.getUrl(_other.name);
    //card_counter++;
    //card_counter++;
}

Card::Card(const Card & _other, bool no_download):  QObject (Q_NULLPTR), name(_other.name), edition(_other.edition), description(_other.description), position(_other.position), is_selected(_other.is_selected), id(_other.id), tapped(_other.tapped), type(_other.type), commander(_other.commander), url_small(_other.url_small), url_big(_other.url_big), cost( _other.cost)
{
    //card_counter++;
    //card_counter++;
}

Card::~Card()
{
    //qDebug() << "Card destructor called " << name;
}

Card::Card() :
	name("empty constructor"), edition("empty constructor"), description("empty constructor"), url_small("empty constructor"), url_big("empty constructor"), id(-1), type(-1), commander(0), position(-1), cost(0)
{
    qDebug() << "Empty Card Constructor was called";
}

Card &Card::operator =(const Card &_other)
{
    name=_other.name;
    edition=_other.edition;
    description=_other.description;
    position=_other.position;
    is_selected=_other.is_selected;
    id=_other.id;
    tapped = _other.tapped;
	type = _other.type;
	url_small = _other.url_small;
	url_big = _other.url_big;
	commander = _other.commander;
	cost = _other.cost;
    return *this;
}

bool Card::operator<(const Card &_other) const
{
	return this->name < _other.name;
}

int Card::costToInt(QString _cost)
{
	int cost = 0;
	QString firstLetter = QString(_cost[0]);
	if(firstLetter=="①" || firstLetter=="②" || firstLetter=="③" || firstLetter=="④" || firstLetter=="⑤" || firstLetter=="⑥" || firstLetter=="⑦" || firstLetter=="⑧" || firstLetter=="⑨")
	{
		if(firstLetter=="①")
		{
			cost += 1;
		}
		else if(firstLetter=="②")
		{
			cost += 2;
		}
		else if(firstLetter=="③")
		{
			cost += 3;
		}
		else if(firstLetter=="④")
		{
			cost += 4;
		}
		else if(firstLetter=="⑤")
		{
			cost += 5;
		}
		else if(firstLetter=="⑥")
		{
			cost += 6;
		}
		else if(firstLetter=="⑦")
		{
			cost += 7;
		}
		else if(firstLetter=="⑧")
		{
			cost += 8;
		}
		else if(firstLetter=="⑨")
		{
			cost += 9;
		}
		_cost = _cost.remove(0,1);
	}
	return cost + _cost.length()/2; //length is bytes and unicode is 2 bytes per char.
}

int Card::TypeStringToEnum(const QString &_type) const
{
	if (_type.toLower().contains("creature"))
	{
		return TYPE_CREATURE;
	}

	if (_type.toLower().contains("artifact"))
	{
		return TYPE_ARTIFICAT;
	}

	if (_type.toLower().contains("enchantment"))
	{
		return TYPE_ENCHANTMENT;
	}

	if (_type.toLower().contains("instant") || _type.toLower().contains("sorcery"))
	{
		return TYPE_SPELL;
	}

	else if (_type.toLower().contains("land"))
	{
		return TYPE_LAND;
	}

	return TYPE_OTHER;
}

int Card::TypeToCommander(const QString &_type) const
{
	return (_type.contains("Legendary") && _type.contains("Creature")) == true ? 1 : 0;
}

void Card::setUrl(QString _name, QString _small, QString _big)
{
    //qDebug() << "Get URL Called:" << _name;
    //qDebug() << _name << "done";
    //qDebug () << "Names: " << name << _name;
    if(name == _name)
    {
        //qDebug() << "Get URL Called:" << _name;

        url_big = _big;
        url_small = _small;
        download_complete = true;
        emit cardChanged();
    }

}
