#ifndef CARD_H
#define CARD_H

#include <QString>
#include <algorithm>

#include "webapi.h"

class Card  : public QObject
{
    Q_OBJECT

    Q_PROPERTY(QString name READ getName CONSTANT)
    Q_PROPERTY(QString edition READ getEdition CONSTANT)
    Q_PROPERTY(QString description READ getDescription CONSTANT)
    Q_PROPERTY(QString url_small READ getUrlSmall NOTIFY cardChanged)
    Q_PROPERTY(QString url_big READ getUrlBig NOTIFY cardChanged)
    Q_PROPERTY(int is_selected READ getSelected WRITE setSelected NOTIFY cardChanged)
    Q_PROPERTY(int position READ getPosition WRITE setPosition NOTIFY cardChanged)
    Q_PROPERTY(int card_id READ getId NOTIFY cardChanged)
    Q_PROPERTY(int tapped READ getTapped WRITE setTapped NOTIFY cardChanged)
	Q_PROPERTY(int card_type READ getType WRITE setType)



public:

    enum
    {
        STATE_CARD_UNTAPPED=0,
        STATE_CARD_TAPPED=1
    };

    enum
    {
        CARD_NOT_SELECTED=0,
        CARD_SELECTED=1
    };

    enum
    {
        POS_PLIB=0,
        POS_PHAND=1,
        POS_PFIELD=2,
        POS_PGRAVE=3,
        POS_PREMOVED=4,
		POS_OLIB=5,
		POS_OHAND=6,
		POS_OFIELD=7,
		POS_OGRAVE=8,
		POS_OREMOVED=9
    };

	enum
	{
		TYPE_LAND=0,
		TYPE_CREATURE=1,
		TYPE_ENCHANTMENT=2,
		TYPE_ARTIFICAT=3,
		TYPE_SPELL=4,
		TYPE_OTHER=5
	};

	Card(const QString& _name, const QString& _edition, const QString& _description, const QString& _type, const QString& _cost, const int _pos = Card::POS_PLIB);
    Card(const QString& _name, const int _pos = Card::POS_PLIB);
    Card(const Card& _other);
    Card(const Card& _other, bool no_download);
    ~Card();
    Card();
    Card& operator =(const Card& _other);
    QString name;
    QString edition;
    QString description;
    QString url_small;
    QString url_big;
    int position = 0; //hand, playfield
    int state = STATE_CARD_UNTAPPED;
    int is_selected = CARD_NOT_SELECTED;
    static int card_counter;
    int id;
    int tapped=0;
    bool download_complete = false;
	int type = 0;
	int commander = 0;
	int cost = 0;

	inline void setType(int _type)
	{
		type = _type;
	}

    static Webapi webapi;
    //Webapi webapi;

    bool operator<(const Card& _other) const;

    inline bool getSelected() const
    {
        return is_selected;
    }

	inline int getType() const
	{
		return type;
	}

    inline void setSelected(const int _state)
    {
        if(_state == CARD_NOT_SELECTED || _state == CARD_SELECTED)
        is_selected = _state;
        emit cardChanged();
    }

    inline void setPosition(const int _pos)
    {
        //if(_state == CARD_NOT_SELECTED || _state == CARD_SELECTED)
        position = _pos;
        emit cardChanged();
    }

    inline void setTapped(const int _tapped)
    {
        //if(_state == CARD_NOT_SELECTED || _state == CARD_SELECTED)
        tapped = _tapped;
        emit cardChanged();
    }


    inline QString getName() const
    {
        return name;
    }
    inline QString getEdition()  const
    {
        return edition;
    }
    inline QString getDescription() const
    {
        return description;
    }
    inline QString getUrlSmall() const
    {
        return url_small;
    }
    inline QString getUrlBig() const
    {
        return url_big;
    }
    inline int getPosition() const
    {
        return position;
    }

    inline int getId() const
    {
        return id;
    }

    inline int getTapped() const
    {
        return tapped;
    }

    inline operator QString() const
    {
        //return name + "(" + QString::number(id) + ")" + " | " + edition + " | " + description + /*" | " + url_small + "| " + url_big */ + " | Pos: " + QString::number(position);
        return name + "("+QString::number(position)+")";
    }

	static int costToInt(QString _cost);

	int TypeStringToEnum(const QString& _type) const;
	int TypeToCommander(const QString& _type) const;

public slots:
    void setUrl(QString _name, QString _small, QString _big);

signals:
    void cardChanged();
};

Q_DECLARE_METATYPE(Card)

#endif // CARD_H
