#include "deckmodel.h"

Deckmodel::Deckmodel(QObject *parent)
    : QAbstractListModel(parent), cards( *(new QVector<Card>())), modelPosition(Gamestate::POSITION_PREMOVED)
{
    //delete &cards;
}

Deckmodel::Deckmodel(QObject *parent, QVector<Card>& _cards, const int _modelPosition) :
    QAbstractListModel(parent), cards(_cards), modelPosition(_modelPosition)
{
    //cards.last().setSelected(Card::CARD_SELECTED);
    updateConnections();
}

void Deckmodel::setCards(QVector<Card> &_cards)
{
    /*
    if(referenceWorkaroundHackDelete == false)
    {
        QVector<Card>* temp = &cards;
        cards = _cards;
        delete temp;
        referenceWorkaroundHackDelete = true;
        qDebug() << cards;
    }
    else
    {
        cards = _cards;
    }
    */
    cards = _cards;
    updateConnections();
	//qDebug() << cards;
}

void Deckmodel::sortDeck()
{
	if(cards.size() == 0)
	{
		return;
	}

	emit layoutAboutToBeChanged();

	for(uint a = 0; a < cards.size(); a++)
	{
		for(uint b = a+1; b < cards.size(); b++)
		{
			if(cards[a].type > cards[b].type)
			{
				Card temp = cards[a];
				cards[a] = cards[b];
				cards[b] = temp;
			}
			else if (cards[a].type == cards[b].type)
			{
				if(cards[a].name > cards[b].name)
				{
					Card temp = cards[a];
					cards[a] = cards[b];
					cards[b] = temp;
				}
			}
		}
	}

}

void Deckmodel::sortDeckIncludingLayoutSignal()
{
	sortDeck();
	emit layoutChanged();
}

void Deckmodel::untapDeck()
{
	beginResetModel();
	for (uint i = 0; i < cards.size();i++)
	{
		cards[i].tapped = 0;
	}
	endResetModel();
}

void Deckmodel::sortDeckByCommander()
{
	qDebug() << "Called " << "sortDeckByCommander" << " with Cardnumer: " << cards.size();

	if(cards.size() == 0)
	{
		return;
	}

	emit layoutAboutToBeChanged();

	for(uint a = 0; a < cards.size(); a++)
	{
		for(uint b = a+1; b < cards.size(); b++)
		{
			if(cards[a].commander < cards[b].commander)
			{
				Card temp = cards[a];
				cards[a] = cards[b];
				cards[b] = temp;
			}
			else if (cards[a].commander == cards[b].commander)
			{
				if(cards[a].type > cards[b].type)
				{
					Card temp = cards[a];
					cards[a] = cards[b];
					cards[b] = temp;
				}
				else if(cards[a].type == cards[b].type && cards[a].name > cards[b].name)
				{
					Card temp = cards[a];
					cards[a] = cards[b];
					cards[b] = temp;
				}

			}

		}
	}
	emit layoutChanged();

}

int Deckmodel::rowCount(const QModelIndex &parent) const
{
    // For list models only the root node (an invalid parent) should return the list's size. For all
    // other (valid) parents, rowCount() should return 0 so that it does not become a tree model.
    if (parent.isValid())
        return 0;

    //qDebug() << "rowCount: " << cards.size();
    return cards.size();
    // FIXME: Implement me!
}

QVariant Deckmodel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    // FIXME: Implement me!

    if(index.row() > cards.size())
    {
        return QVariant();
    }

    const QString name (cards[index.row()].name );
    const QString desc (cards[index.row()].description );
    const QString edit  (cards[index.row()].edition );
    const QString urlsmall (cards[index.row()].url_small );
    const QString urlbig (cards[index.row()].url_big);
    const int is_selected (cards[index.row()].is_selected);
    const int position (cards[index.row()].position);
    const int id (cards[index.row()].getId());
    const int tapped (cards[index.row()].getTapped());
	const int type (cards[index.row()].getType());

    switch(role)
    {
        case NameRole: return(QVariant::fromValue(name)); break;
        case EditionRole: return(QVariant::fromValue(edit)); break;
        case DescriptionRole: return(QVariant::fromValue(desc)); break;
        case UrlSmallRole: return(QVariant::fromValue(urlsmall)); break;
        case UrlBigRole: return(QVariant::fromValue(urlbig)); break;
        case IsSelectedRole: return (QVariant::fromValue(is_selected)); break;
        case PositionRole: return (QVariant::fromValue(position)); break;
        case IdRole: return (QVariant::fromValue(id)); break;
        case TappedRole: return (QVariant::fromValue(tapped)); break;
		case CardType: return (QVariant::fromValue(type)); break;
    }

    return QVariant();
}

bool Deckmodel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    //qDebug() << "setData" << index << " | " << value << " | " << role;
    if (data(index, role) != value)
    {
        // FIXME: Implement me!

        switch(role)
        {
            case IsSelectedRole:
                //qDebug() << "Selected: "  << cards[index.row()].name << ": " << cards[index.row()].is_selected << " to value " << value.toInt();
                cards[index.row()].is_selected = value.toInt();
                break;

            case PositionRole:
				//qDebug() << "Position: "  << cards[index.row()].name << ": " << cards[index.row()].position << " to value " << value.toInt();
                cards[index.row()].position = value.toInt();
                //cards[index.row()].tapped = 0; //untap if you move card, maybe stupid for battlefield but good for hands
                emit MoveCard(modelPosition,value.toInt(),cards[index.row()].id);
                break;

            case TappedRole:
                qDebug() << "Tapped: "  << cards[index.row()].tapped << ": " << cards[index.row()].tapped << " to value " << value.toInt();
                cards[index.row()].tapped = value.toInt();
                break;
        }

        emit dataChanged(index, index, QVector<int>() << role);
        return true;
    }
    return false;
}

Qt::ItemFlags Deckmodel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return Qt::NoItemFlags;

    return Qt::ItemIsEditable; // FIXME: Implement me!
}

bool Deckmodel::insertRows(int row, int count, const QModelIndex &parent)
{
    beginInsertRows(parent, row, row + count - 1);
    // FIXME: Implement me!
    endInsertRows();
    return true;
}

bool Deckmodel::removeRows(int row, int count, const QModelIndex &parent)
{
    beginRemoveRows(parent, row, row + count - 1);
    // FIXME: Implement me!
    endRemoveRows();
    return true;
}

QHash<int, QByteArray> Deckmodel::roleNames() const
{
    QHash<int, QByteArray> names;
    names[NameRole] = "name";
    names[EditionRole] = "edition";
    names[DescriptionRole] = "description";
    names[UrlSmallRole] = "urlsmall";
    names[UrlBigRole] = "urlbig";
    names[IsSelectedRole] = "is_selected";
    names[PositionRole] = "position";
    names[IdRole] = "card_id";
    names[TappedRole] = "tapped";
	names[CardType] = "card_type";
    return names;
}

void Deckmodel::updateConnections()
{
    for(int i = 0; i < cards.size();i++)
    {
        //const Card& temp = cards[i];
        connect(&cards[i], SIGNAL(cardChanged()) , this, SLOT(sendUrlFinishedSignal() ));
        //connect(this, SIGNAL(destroyed(QObject*)), this, SLOT(sendUrlFinishedSignal() ));
    }
}


void Deckmodel::sendUrlFinishedSignal()
{
    //emit QAbstractListModel::dataChanged(index(0), index(0));
    //qDebug() << "Modelposition:" << modelPosition << "Rowsize" << rowCount() << " Cardsize: " << cards.size();
	//qDebug() << "Url Finished Signal send";
    emit QAbstractListModel::layoutChanged();
}

void Deckmodel::externalModelChange(int _pos)
{
    if(modelPosition == _pos)
    {
        //for each element insert it into model, delete stuff in it before
        insertRow(0);
        emit QAbstractListModel::layoutChanged();

    }
}
