#ifndef DECKMODEL_H
#define DECKMODEL_H

#include <QAbstractListModel>
#include "card.h"
#include "vector"
#include "gamestate.h"

class Deckmodel : public QAbstractListModel
{
    Q_OBJECT
	Q_PROPERTY(int size READ getSize)

	friend class Ai;

public:

        enum DeckmodelRoles {
            NameRole = 0,
            EditionRole  = 1,
            DescriptionRole  = 2,
            UrlSmallRole  = 3,
            UrlBigRole = 4,
            IsSelectedRole = 5,
            PositionRole = 6,
            IdRole = 7,
			TappedRole = 8,
			CardType = 9
        };


    bool referenceWorkaroundHackDelete = false;

    explicit Deckmodel(QObject *parent = nullptr);
    explicit Deckmodel(QObject *parent, QVector<Card>& _cards, const int _modelPosition);

    void setCards(QVector<Card>& _cards);


    // Basic functionality:
	int rowCount(const QModelIndex &parent = QModelIndex()) const override;

    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;

    // Editable:
    bool setData(const QModelIndex &index, const QVariant &value,
                 int role = Qt::EditRole) override;

    Qt::ItemFlags flags(const QModelIndex& index) const override;

    // Add data:
    bool insertRows(int row, int count, const QModelIndex &parent = QModelIndex()) override;

    // Remove data:
    bool removeRows(int row, int count, const QModelIndex &parent = QModelIndex()) override;

    QHash<int, QByteArray> roleNames() const;

    void updateConnections();

    int modelPosition;


public slots:
	inline int getSize() const
	{
		return cards.size();
	}
    void sendUrlFinishedSignal();
    void externalModelChange(int _pos);
	Q_INVOKABLE void sortDeck();
	Q_INVOKABLE void sortDeckIncludingLayoutSignal();
	Q_INVOKABLE void untapDeck();
	Q_INVOKABLE void sortDeckByCommander();


signals:
	void MoveCard(int source, int target, int cardid);

private:
    QVector<Card>& cards;

};

#endif // DECKMODEL_H
