#ifndef GAMESTATE_H
#define GAMESTATE_H

#include <QVector>
#include <QObject>
#include <QStandardItemModel>
#include <QStringList>
#include <QtAlgorithms>
#include <random>
#include <algorithm>
#include <iterator>
#include <iostream>
#include <QFile>
#include <QEventLoop>
#include <QTimer>
#include "life.h"
#include "card.h"

class Ai;

class Gamestate : public QObject
{
    Q_OBJECT

    Q_PROPERTY(QStandardItemModel* model READ model)
    Q_PROPERTY(int player_life READ getPlayerLife WRITE setPlayerLife NOTIFY playerLifeChanged)
    Q_PROPERTY(int opponent_life READ getOpponentLife WRITE setOpponentLife NOTIFY opponentLifeChanged)
	Q_PROPERTY(QString chat_text READ getChatText WRITE setChatText NOTIFY chatTextChanged)
	Q_PROPERTY(int battleFieldIsSorted READ isSorted WRITE sortBattleField NOTIFY chatTextChanged)
	//getOpponentLife
    //getPlayerLife

    Q_DISABLE_COPY(Gamestate)


public:

    enum
    {
        POSITION_PLIB = 0,
        POSITION_PHAND = 1,
        POSITION_PFIELD = 2,
        POSITION_PGRAVE = 3,
        POSITION_PREMOVED = 4,
        POSITION_OLIB = 5,
        POSITION_OHAND = 6,
        POSITION_OFIELD = 7,
        POSITION_OGRAVE = 8,
        POSITION_OREMOVED = 9
    };

    explicit Gamestate(QObject* parent, QString _filename = "commander.deck");
    virtual ~Gamestate();

    QStandardItemModel* model()
    {
        return player_hand_model;
    }

    Q_SLOT void addCard(const Card& _card);

	QString chat_text;
    QVector<Card> player_hand;
    QVector<Card> player_lib;
    QVector<Card> player_grave;
    QVector<Card> player_exile;
    QVector<Card> player_field;
    QVector<Card> opponent_hand;
    QVector<Card> opponent_lib;
    QVector<Card> opponent_grave;
    QVector<Card> opponent_exile;
    QVector<Card> opponent_field;
    QStringList dataList;
    QStringList player_tokens;
    QStringList opponent_tokens;
	QString lastCardDrawn;
    Life life;
	int player_life = 40;
	int opponent_life = 40;

    QVector<Card>& getVectorFromId(int _id);

    void loadPlayerDeck(const QString& _name);
    void loadEnemyDeck(const QString& _name);

    void loopOverCardsAndPutThemInTheVectorAccordingToItsPosition(int _source);
	static QString positionEnumToString(int _enum);

    public slots:

	void setChatText(const QString& _text);
    void tokenPlayerChanged(int index, const QString text);
    void createPlayerToken();
    void deletePlayerToken(const QString& text);
    bool isReady();
    void moveCardPosition(int _source, int _target, int _id);
    void shufflePlayerLib();
    void shuffleEnemyLib();
    void drawCardPlayer();
    void setPositionVariableAccordingToContainer();
    void handleGameStateChangedPreSignal(int i);
    void sortPlayerDeck();
	void sortBattleField(int i);

	inline QString getLastCardDrawn() const
	{
		return lastCardDrawn;
	}

    inline int getPlayerLife() const
    {
        return player_life;
    }
    inline void setPlayerLife(int _life)
    {
        player_life = _life;
    }

    inline int getOpponentLife() const
    {
        return opponent_life;
    }
    inline void setOpponentLife(int _life)
    {
        opponent_life = _life;
    }

	inline QString getChatText()
	{
		return chat_text;
	}

	inline int isSorted()
	{
		return 0;
	}

signals:
    void gameStateChanged(int);
    void gameStateChangedPreSignal(int);
    void playerLifeChanged(int _life);
    void opponentLifeChanged(int _life);
    void playerDeckReady();
    void tokensChanged();
	void chatTextChanged(QString _text);
	void chatInformation(QVariant cardname);
	void sortedBattlefield();

private:
    QStandardItemModel* player_hand_model;
};

#endif // GAMESTATE_H
