#include <QtCore>
#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QQmlContext>
#include <QQuickWindow>
#include <QPixmap>
#include "gamestate.h"
#include "deckmodel.h"
#include "connector.h"
#include "ai.h"
//Webapi Card::webapi;


Webapi Card::webapi;
int Card::card_counter = 0;

int main(int argc, char *argv[])
{
    //Card big("Birds of Paradise","Alpha","Tap for any mana");


    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    QGuiApplication app(argc, argv);
	app.setWindowIcon(QIcon(":/icon.png"));


    //fix the model this way https://wiki.qt.io/How_to_Use_a_Custom_Class_in_C%2B%2B_Model_and_QML_View

	QQmlApplicationEngine engine;

    if(argc != 2)
    {
        qDebug() << "Crash! Magicnet needs one parameter. Example: magicnet mydeck.deck";
        return 1;
    }

    qDebug() << "argv[0] " << argv[0] << " | argv[1] " << argv[1];
    Gamestate gamestate(Q_NULLPTR, argv[1]);

    Connector connector(engine,gamestate);

    Deckmodel deckmodel (Q_NULLPTR,gamestate.player_hand, Gamestate::POSITION_PHAND);
    Deckmodel fieldmodel (Q_NULLPTR,gamestate.player_field, Gamestate::POSITION_PFIELD);
    Deckmodel libmodel (Q_NULLPTR,gamestate.player_lib, Gamestate::POSITION_PLIB);
    Deckmodel gravemodel (Q_NULLPTR,gamestate.player_grave, Gamestate::POSITION_PGRAVE);
    Deckmodel exilemodel (Q_NULLPTR,gamestate.player_exile, Gamestate::POSITION_PREMOVED);

	Deckmodel opponentFieldModel (Q_NULLPTR,gamestate.opponent_field, Gamestate::POSITION_OFIELD);
	Deckmodel opponentGraveModel (Q_NULLPTR,gamestate.opponent_grave, Gamestate::POSITION_OGRAVE);
	Deckmodel opponentHandModel (Q_NULLPTR,gamestate.opponent_hand, Gamestate::POSITION_OHAND);
	Deckmodel opponentLibModel (Q_NULLPTR,gamestate.opponent_lib, Gamestate::POSITION_OLIB);
	Deckmodel opponentExileModel (Q_NULLPTR,gamestate.opponent_exile, Gamestate::POSITION_OREMOVED);

	Ai ai(Q_NULLPTR,&fieldmodel,&opponentFieldModel,&opponentHandModel,&opponentLibModel,&gamestate);

    //connector.connectCardMove(gamestate,deckmodel);

    QObject::connect(&deckmodel, &Deckmodel::MoveCard,&gamestate,&Gamestate::moveCardPosition);
    QObject::connect(&gamestate, &Gamestate::gameStateChanged,&deckmodel,&Deckmodel::sendUrlFinishedSignal);

    QObject::connect(&fieldmodel, &Deckmodel::MoveCard,&gamestate,&Gamestate::moveCardPosition);
    QObject::connect(&gamestate, &Gamestate::gameStateChanged,&fieldmodel,&Deckmodel::sendUrlFinishedSignal);

    QObject::connect(&libmodel, &Deckmodel::MoveCard,&gamestate,&Gamestate::moveCardPosition);
    QObject::connect(&gamestate, &Gamestate::gameStateChanged,&libmodel,&Deckmodel::sendUrlFinishedSignal);

    QObject::connect(&gravemodel, &Deckmodel::MoveCard,&gamestate,&Gamestate::moveCardPosition);
    QObject::connect(&gamestate, &Gamestate::gameStateChanged,&gravemodel,&Deckmodel::sendUrlFinishedSignal);

    QObject::connect(&exilemodel, &Deckmodel::MoveCard,&gamestate,&Gamestate::moveCardPosition);
    QObject::connect(&gamestate, &Gamestate::gameStateChanged,&exilemodel,&Deckmodel::sendUrlFinishedSignal);


	QObject::connect(&opponentFieldModel, &Deckmodel::MoveCard,&gamestate,&Gamestate::moveCardPosition);
	QObject::connect(&gamestate, &Gamestate::gameStateChanged,&opponentFieldModel,&Deckmodel::sendUrlFinishedSignal);

	QObject::connect(&opponentHandModel, &Deckmodel::MoveCard,&gamestate,&Gamestate::moveCardPosition);
	QObject::connect(&gamestate, &Gamestate::gameStateChanged,&opponentHandModel,&Deckmodel::sendUrlFinishedSignal);

	QObject::connect(&opponentGraveModel, &Deckmodel::MoveCard,&gamestate,&Gamestate::moveCardPosition);
	QObject::connect(&gamestate, &Gamestate::gameStateChanged,&opponentGraveModel,&Deckmodel::sendUrlFinishedSignal);

	QObject::connect(&opponentLibModel, &Deckmodel::MoveCard,&gamestate,&Gamestate::moveCardPosition);
	QObject::connect(&gamestate, &Gamestate::gameStateChanged,&opponentLibModel,&Deckmodel::sendUrlFinishedSignal);

	QObject::connect(&opponentExileModel, &Deckmodel::MoveCard,&gamestate,&Gamestate::moveCardPosition);
	QObject::connect(&gamestate, &Gamestate::gameStateChanged,&opponentExileModel,&Deckmodel::sendUrlFinishedSignal);


    QObject::connect(&gamestate, &Gamestate::gameStateChangedPreSignal,&gamestate,&Gamestate::handleGameStateChangedPreSignal);



    engine.rootContext()->setContextProperty("Gamestate", &deckmodel);
    engine.rootContext()->setContextProperty("Playerfield", &fieldmodel);
    engine.rootContext()->setContextProperty("Playerlib", &libmodel);
    engine.rootContext()->setContextProperty("Playergrave", &gravemodel);
    engine.rootContext()->setContextProperty("Playerexile", &exilemodel);

    engine.rootContext()->setContextProperty("Playertokens", QVariant::fromValue(gamestate.player_tokens));
    engine.rootContext()->setContextProperty("Opponenttokens", QVariant::fromValue(gamestate.opponent_tokens));
	engine.rootContext()->setContextProperty("OpponentField", &opponentFieldModel);
	engine.rootContext()->setContextProperty("OpponentHand", &opponentHandModel);
	engine.rootContext()->setContextProperty("OpponentLib", &opponentLibModel);
	engine.rootContext()->setContextProperty("OpponentGrave", &opponentGraveModel);
	engine.rootContext()->setContextProperty("OpponentExile", &opponentExileModel);

    engine.rootContext()->setContextProperty("Lifestate", &gamestate.life);
	engine.rootContext()->setContextProperty("Ai", &ai);


	engine.load(QUrl(QStringLiteral("qrc:/main.qml")));
	if (engine.rootObjects().isEmpty())
		return -1;



	//Game info connections:

	QObject *topLevel = engine.rootObjects().value(0);
	QQuickWindow *window = qobject_cast<QQuickWindow *>(topLevel);
	QObject::connect(&gamestate,
					 SIGNAL(chatInformation(QVariant)),
					 window,
					 SLOT(handleInformationSignalsForChat(QVariant)));

    QObject *item = engine.rootObjects().first();

    QObject::connect(item, SIGNAL(shufflePlayerLib()),
                         &gamestate, SLOT(shufflePlayerLib()));

    QObject::connect(item, SIGNAL(drawCard()),
                         &gamestate, SLOT(drawCardPlayer()));

    QObject::connect(item, SIGNAL(sortPlayerDeck()),&gamestate, SLOT(sortPlayerDeck()));

    QObject::connect(item, SIGNAL(deletePlayerToken(QString)),&gamestate, SLOT(deletePlayerToken(QString)));


    //Note: There is no way for the view to know that the contents of a QStringList have changed. If the QStringList changes, it will be necessary to reset the model by calling QQmlContext::setContextProperty() again.
    //only important for the the opponent tokens
    QObject::connect(item, SIGNAL(tokenPlayerChanged(int, QString)),&gamestate, SLOT(tokenPlayerChanged(int,QString)));
    QObject::connect(item, SIGNAL(createPlayerToken()),&gamestate, SLOT(createPlayerToken()));
    QObject::connect(&gamestate,&Gamestate::tokensChanged,&connector,&Connector::updateTokens);

    //return app.exec();

    app.exec();
    int x = 2;
    return x-2;
}
