import QtQuick 2.9
import QtQuick.Window 2.3
import QtQuick.Layouts 1.3
import QtQuick.Controls 2.2
import QtMultimedia 5.9


Window
{
        visible: true
        width: 1880
        height: Screen.desktopAvailableHeight
        title: qsTr("Magic Deck Tester")

        property int card_size_x: 80
        property int card_size_y: 120
        property int table_width: width - width/4.5

        property int hand_size_y: card_size_y
        property int divider_size_y: 4
        property int table_height: (Screen.desktopAvailableHeight-(hand_size_y*2)-(divider_size_y))/2-30
        property int card_id_counter: 0


        signal shufflePlayerLib();
        signal drawCard();
        signal sortPlayerDeck();
        signal tokenPlayerChanged(int index, string text);
        signal createPlayerToken();
        signal deletePlayerToken(string desc);

        id: mainwindow

        function handleInformationSignalsForChat(text)
        {
            chat_text.updateText(text)
        }


        function hoverOverCard(url)
        {
            cardViewWidget.url=url;
        }

        SoundEffect
        {
            id: playSound
            source  : "http://foto.teoteater.ee/muusika/1001%20Sound%20Effects/Office%20Supplies/Paper%20Rustle.wav"
            volume: 0.2
        }

        SoundEffect
        {
            id: drawSound
            //source  : "http://www.chiptape.com/chiptape/sounds/medium/Minimize1.wav"
            source  : "http://ricostardeluxe.free.fr/PUNCH1.wav"
            volume: 0.05
        }

        RowLayout
        {
            id: gui_seperator

            GridLayout
            {
                id: decks_and_life
                width: 200
                Layout.fillHeight: true
                rows: 4
                columns: 1
                Rectangle
                {
                    Layout.fillHeight: true
                    Layout.fillWidth: true
                    color: "#ff3b3b"

                    ColumnLayout
                    {
                        Button
                        {
                            id: aiTurn
                            text: "AI Turn"
                            onClicked:
                            {
                                chat_text.updateText("===START Ai Turn===")
                                //Ai.AiActionRandom();
                                Ai.AiActionPlayLandLoopUsefulCard();
                                oppent_lib_size.updateText()
                                chat_text.updateText("=== END Ai Turn ===")
                            }
                        }

                        Rectangle
                        {
                            color: "#BABABA"

                            Layout.fillWidth: true
                            height: 50

                            border.color: "black";
                            border.width: 1

                            Text
                            {

                                anchors.horizontalCenter: parent.horizontalCenter
                                anchors.verticalCenter: parent.verticalCenter
                                font.pointSize: 16

                                id: oppent_lib_size
                                text: OpponentLib.size

                                function updateText()
                                {
                                    text = OpponentLib.size
                                }
                            }
                        }

                        ListView
                        {
                            id:grave_op
                            Layout.minimumHeight: card_size_y
                            Layout.minimumWidth: parent.width
                            clip:true

                            ScrollBar.vertical: ScrollBar
                            {
                                active: true
                                policy: ScrollBar.AlwaysOn
                            }

                            model: OpponentGrave

                            delegate: OpponentDeckCard
                            {
                                cardname: model.name
                                card_id: model.card_id
                                position: model.position
                                width: parent.width-10

                                function updateModel(state)
                                {
                                    model.position = position
                                }

                                Component.onCompleted:
                                {
                                    clickedOnCard.connect(updateModel)
                                }
                            }
                        }

                        ListView
                        {
                            id:op_exile
                            Layout.minimumHeight: card_size_y
                            Layout.minimumWidth: parent.width
                            clip:true


                            ScrollBar.vertical: ScrollBar
                            {
                                active: true
                                policy: ScrollBar.AlwaysOn
                            }


                            model: OpponentExile

                            delegate: OpponentDeckCard
                            {
                                cardname: model.name
                                card_id: model.card_id
                                position: model.position
                                width: parent.width-10
                                color: "lightgreen"

                                function updateModel(state)
                                {
                                    model.position = position
                                }

                                Component.onCompleted:
                                {
                                    clickedOnCard.connect(updateModel)
                                }
                            }
                        }
                    }
                }
                RowLayout
                {
                    CheckBox
                    {
                        id: debug
                        hoverEnabled: true
                        text: "Gr"
                        ToolTip.delay: 1000
                        ToolTip.timeout: 5000
                        ToolTip.visible: hovered
                        ToolTip.text: qsTr("Group items like lands together")
                        Layout.maximumWidth: 70
                        checked: table_player.autosort
                        //onClicked: console.log(Screen.desktopAvailableHeight + "|" + table_height)

                        onClicked:
                        {
                            //Gamestate.battleFieldIsSorted = 1
                            //Playerfield.sortDeck()
                            //update()
                            table_player.autosort = checked

                            if(checked)
                            {
                                Playerfield.sortDeckIncludingLayoutSignal()
                            }
                        }

                        //onClicked: Qt.createQmlObject('import QtQuick 2.0; Rectangle {color: "red"; width: '+card_size_x+'; height: '+card_size_y+'}', opponent_hand_layout ,"dynamicSnippet1");
                    }

                    Button
                    {
                        ToolTip.delay: 1000
                        ToolTip.timeout: 5000
                        ToolTip.visible: hovered
                        ToolTip.text: qsTr("Untap everything")

                        Layout.maximumWidth: 40
                        id:untap_button
                        text:"↺"
                        onClicked:
                        {
                            Playerfield.untapDeck();
                        }
                    }

                }

                Rectangle
                {
                    Layout.fillHeight: true
                    Layout.fillWidth: true
                    focus: true

                    Keys.onPressed:
                    {
                        if (event.key == Qt.Key_S)
                        {
                            onPressed: { playSound.play() }
                            mainwindow.shufflePlayerLib();
                        }

                        if(event.key == Qt.Key_D)
                        {
                            onClicked: mainwindow.drawCard();
                            onPressed: { drawSound.play() }
                        }
                        if(event.key == Qt.Key_U)
                        {
                            Playerfield.untapDeck()
                        }
                    }




                    color: "#3d48ff"
                    ColumnLayout
                    {
                        anchors.left: parent.left
                        anchors.right: parent.right


                        Button
                        {
                            Layout.alignment: Qt.AlignTop
                            id: drawCardPlayer
                            text: "Draw"
                            //onClicked: Qt.createQmlObject('import QtQuick 2.0; Rectangle {color: "red"; width: '+card_size_x+'; height: '+card_size_y+'}', opponent_hand_layout ,"dynamicSnippet1");
                            onClicked:
                            {
                                drawSound.play()
                                mainwindow.drawCard();
                            }


                        }

                        Button
                        {
                            Layout.alignment: Qt.AlignTop
                            id: buttonShufflePlayerDeck
                            text: "Shuffle"
                            //onClicked: Qt.createQmlObject('import QtQuick 2.0; Rectangle {color: "red"; width: '+card_size_x+'; height: '+card_size_y+'}', opponent_hand_layout ,"dynamicSnippet1");
                            onClicked:
                            {
                                onPressed: { playSound.play() }
                                console.log("Shuffling Deck Signal sended")
                                mainwindow.shufflePlayerLib();
                                chat_text.updateText("Shuffled deck")
                            }
                            Keys.onPressed:
                            {
                                if (event.key == Qt.Key_S)
                                {
                                    onPressed: { playSound.play() }
                                    console.log("Shuffling Deck Signal sended")
                                    mainwindow.shufflePlayerLib();
                                    chat_text.updateText("Shuffled deck")
                                }
                            }
                        }
                        Row
                        {
                            CheckBox
                            {
                                id: reveal_deck
                                spacing: 1
                                rightPadding: 0
                                bottomPadding: 1
                                leftPadding: -1
                                topPadding: 1
                                font.wordSpacing: 0
                                onClicked:
                                {
                                    if (checkState === Qt.Checked)
                                    {
                                        chat_text.updateText("Started search through deck")
                                    }
                                    else
                                    {
                                        chat_text.updateText("Ended search through deck")
                                    }

                                }
                            }
                            Button
                            {
                                id: sort_deck
                                text:"Sort"
                                rightPadding: 3
                                leftPadding: 4
                                topPadding: 4
                                font.pointSize: 7
                                width:45
                                onClicked:
                                {
                                    mainwindow.sortPlayerDeck();
                                    chat_text.updateText("Sorted deck")
                                }
                            }
                            Button
                            {
                                id: sort_deck_leg
                                text:"Type"
                                rightPadding: 2
                                leftPadding: 4
                                topPadding: 4
                                font.pointSize: 6
                                width:30
                                onClicked:
                                {
                                    Playerlib.sortDeckByCommander();
                                    chat_text.updateText("Sorted deck by Legendary")
                                }
                            }
                        }

                        ListView
                        {
                            id:lib_player
                            Layout.minimumHeight: card_size_y
                            Layout.minimumWidth: parent.width
                            /*Layout.fillWidth: true
                            Layout.fillHeight: true
                            anchors.fill: parent*/
                            clip:true


                            ScrollBar.vertical: ScrollBar
                            {
                                active: true
                                policy: ScrollBar.AlwaysOn

                                //Component.onCompleted: x = 1

                            }

                            model: Playerlib

                            delegate: Deckcard
                            {
                                //cardname: model.name


                                cardname: reveal_deck.checked ? model.name : "Card"
                                card_id: model.card_id
                                position: model.position
                                color: "lightgray"
                                width:parent.width-10

                                function updateModel(state)
                                {
                                    //console.log("New state " + state);
                                    //model.is_selected = is_selected
                                    model.position = position
                                }

                                Component.onCompleted:
                                {
                                    clickedOnCard.connect(updateModel)
                                    //model.is_selected = true;
                                }
                            }


                        }

                        ListView
                        {
                            id:lib_grave
                            Layout.minimumHeight: card_size_y
                            Layout.minimumWidth: parent.width
                            /*Layout.fillWidth: true
                            Layout.fillHeight: true
                            anchors.fill: parent*/
                            clip:true


                            ScrollBar.vertical: ScrollBar
                            {
                                active: true
                                policy: ScrollBar.AlwaysOn

                                //Component.onCompleted: x = 1

                            }


                            model: Playergrave

                            delegate: Deckcard
                            {
                                cardname: model.name
                                card_id: model.card_id
                                position: model.position
                                width: parent.width-10

                                function updateModel(state)
                                {
                                    //console.log("New state " + state);
                                    //model.is_selected = is_selected
                                    model.position = position
                                }

                                Component.onCompleted:
                                {
                                    clickedOnCard.connect(updateModel)
                                    //model.is_selected = true;
                                }
                            }
                        }

                        ListView
                        {
                            id:player_exile
                            Layout.minimumHeight: card_size_y
                            Layout.minimumWidth: parent.width
                            /*Layout.fillWidth: true
                            Layout.fillHeight: true
                            anchors.fill: parent*/
                            clip:true


                            ScrollBar.vertical: ScrollBar
                            {
                                active: true
                                policy: ScrollBar.AlwaysOn

                                //Component.onCompleted: x = 1

                            }


                            model: Playerexile

                            delegate: Deckcard
                            {
                                cardname: model.name
                                card_id: model.card_id
                                position: model.position
                                width: parent.width-10
                                color: "lightgreen"

                                function updateModel(state)
                                {
                                    //console.log("New state " + state);
                                    //model.is_selected = is_selected
                                    model.position = position
                                }

                                Component.onCompleted:
                                {
                                    clickedOnCard.connect(updateModel)
                                    //model.is_selected = true;
                                }
                            }


                        }
                    }
                }
            }



            GridLayout
            {
                rows: 5
                columns: 1
                id: eveything_layout


                Layout.fillHeight: true
                Layout.fillWidth: true

                RowLayout
                {
                    id: opponent_hand_layout
                    width: table_width
                    Layout.minimumWidth: 20
                    transformOrigin: Item.TopLeft
                    Layout.minimumHeight: card_size_y
                    Layout.fillHeight: false
                    Layout.fillWidth: true

                    GridView
                    {
                        id:opponent_hand_cards_view
                        Layout.minimumHeight: card_size_y
                        Layout.minimumWidth: card_size_x
                        width: table_height
                        model: OpponentHand
                        anchors.fill: parent

                        delegate: OpponentCard
                        {
                            url: model.urlsmall
                            urlbig: model.urlbig
                            cardname: model.name
                            is_selected: model.is_selected
                            position: model.position
                            card_id: model.card_id

                            function updateModel(state)
                            {
                                model.position = position

                            }

                            Component.onCompleted:
                            {
                                hoveredOnCard.connect(hoverOverCard)
                                selectionChanged.connect(updateModel)
                                clickedOnCard.connect(updateModel)
                            }
                        }
                    }
                }

                GridView
                {
                        id:table_opponent
                        Layout.minimumHeight: card_size_y
                        Layout.minimumWidth: table_width - table_width/6
                        //Layout.fillWidth: true
                        Layout.fillHeight: true
                        //anchors.fill: parent
                        width: table_width - table_width/6
                        height: table_height
                        anchors.leftMargin: 25
                        property bool autosort: true

                        cellHeight: card_size_y+2

                        model: OpponentField

                            delegate: OpponentCard
                            {
                                url: model.urlsmall
                                urlbig: model.urlbig
                                cardname: model.name
                                is_selected: model.is_selected
                                position: model.position
                                card_id: model.card_id
                                tapped: model.tapped
                                card_type: model.card_type

                                function updateModel(state)
                                {
                                    model.tapped = tapped;
                                    model.position = position;
                                }

                                Component.onCompleted:
                                {
                                    hoveredOnCard.connect(hoverOverCard)
                                    clickedOnCard.connect(updateModel)
                                    tapChanged.connect(updateModel)
                                }
                            }
                }


                RowLayout
                {
                    id: divider_layout
                    Layout.fillHeight: false
                    Layout.fillWidth: true
                    width: table_width

                    Rectangle {
                        id: devider
                        height: divider_size_y
                        color: "#ceadad"
                        Layout.fillWidth: true
                    }
                }


                Rectangle
                {
                    Layout.fillWidth: true
                    Layout.fillHeight: true

                    width: table_width
                    height: table_height

                    Rectangle
                    {
                        id: table_player_background
                        color: "lightblue"
                        z: 0
                        opacity: 0.1
                        Layout.fillWidth: true
                        Layout.fillHeight: true

                        width: table_width
                        height: table_height
                    }

                    GridView
                    {
                        id:table_player
                        Layout.minimumHeight: card_size_y
                        Layout.minimumWidth: table_width - table_width/6
                        //Layout.fillWidth: true
                        Layout.fillHeight: true
                        //anchors.fill: parent
                        width: table_width - table_width/6
                        height: table_height
                        anchors.leftMargin: 25
                        property bool autosort: true

                        cellHeight: card_size_y+2

                        model: Playerfield

                            delegate: Card
                            {
                                url: model.urlsmall
                                urlbig: model.urlbig
                                cardname: model.name
                                is_selected: model.is_selected
                                position: model.position
                                card_id: model.card_id
                                tapped: model.tapped
                                card_type: model.card_type

                                function updateModel(state)
                                {
                                    console.log("Playerfield updateModel");
                                    model.tapped = tapped;
                                    model.is_selected = is_selected;
                                    model.position = position;
                                }

                                Component.onCompleted:
                                {
                                    hoveredOnCard.connect(hoverOverCard)
                                    selectionChanged.connect(updateModel)
                                    tapChanged.connect(updateModel)

                                    if(table_player.autosort == true)
                                    {
                                        Playerfield.sortDeck()
                                    }
                                }
                            }
                    }

                    GridView
                    {
                        id:table_player_tokens
                        Layout.minimumHeight: card_size_y
                        Layout.minimumWidth: table_width/6

                        anchors.fill: parent
                        width: table_width/6
                        height: table_height
                        anchors.leftMargin: table_width - table_width/6
                        cellHeight: card_size_y+2

                        model: Playertokens

                            delegate: Rectangle
                            {
                                width: card_size_x
                                height: card_size_y
                                color: "red"
                                property int indexOfThisDelegate: index


                                TextArea
                                {
                                    id:tokentext
                                    text: model.modelData
                                    wrapMode: Text.WrapAnywhere
                                    anchors.fill: parent

                                    onTextChanged:
                                    {
                                        //console.log("Text changed");
                                        mainwindow.tokenPlayerChanged(parent.indexOfThisDelegate,text);
                                    }

                                    MouseArea
                                    {
                                        anchors.fill: parent
                                        acceptedButtons: Qt.RightButton

                                        onClicked:
                                        {
                                            //console.log("mainwindow.deletePlayerToken("+parent.text+")");
                                            mainwindow.deletePlayerToken(parent.text);
                                        }
                                    }
                                }

                            }

                    }


                }

                RowLayout
                {
                        id: player_hand_layout
                        width: table_width
                        Layout.fillHeight: false
                        Layout.fillWidth: true
                        Layout.minimumHeight: card_size_y

                        GridView
                        {
                            id:player_hand_cards_view
                            Layout.minimumHeight: card_size_y
                            Layout.minimumWidth: card_size_x
                            width: table_height
                            model: Gamestate
                            anchors.fill: parent

                            delegate: Card
                            {
                                url: model.urlsmall
                                urlbig: model.urlbig
                                cardname: model.name
                                is_selected: model.is_selected
                                position: model.position
                                card_id: model.card_id

                                /*
                                    //not working yet
                                    Binding
                                    {
                                        target: Gamestate.model.is_selected
                                        property: "is_selected"
                                        value: is_selected
                                    }
                                */

                                function updateModel(state)
                                {
                                    //console.log("player_hand_layout updateModel");
                                    model.tapped = tapped
                                    model.is_selected = is_selected
                                    model.position = position

                                }

                                Component.onCompleted:
                                {
                                    hoveredOnCard.connect(hoverOverCard)
                                    selectionChanged.connect(updateModel)
                                    //model.is_selected = true;
                                }
                            }
                        }
                }
            }

            GridLayout
            {
                id: card_information
                width: 200
                Layout.fillHeight: true
                rows: 3
                columns: 1
                CardView
                {
                    id:cardViewWidget
                }

                /*
                Row
                {
                    Rectangle
                    {

                        Layout.fillWidth: true
                        width: 100;
                        height: 40;
                        id:player_life_box
                        color:"green"
                        Text
                        {
                            id:player_life_text
                            text:"666"
                        }
                    }
                    Rectangle
                    {

                        Layout.fillWidth: true
                        width: 100;
                        height: 40;
                        id:opponent_life_box
                        color:"red"
                        Text
                        {
                            id:opponent_life_text
                            text:"999"
                        }
                    }
                }
                */

                Row
                {
                    clip: true
                    Infobox
                    {

                    }
                    Button
                    {
                        text: "Token"
                        onClicked:
                        {
                            mainwindow.createPlayerToken();
                            chat_text.updateText("Created Token");
                        }
                    }

                }

                Rectangle
                {
                    id: chat
                    Layout.fillHeight: true
                    Layout.fillWidth: true
                    color: "#DDDDFF"

                    /*
                    Column
                    {
                        Row
                        {
                            Rectangle
                            {
                                Layout.fillHeight: true
                                Layout.fillWidth: true
                                anchors.fill: parent;
                                width: 100;
                                height: 100;
                                id:player_life_box
                                color:"green"
                                Text
                                {
                                    id:player_life_text
                                    text:"666"
                                }
                            }
                        }*/

                    Flickable
                    {
                        id: flickable
                        anchors.fill: parent
                        width: parent.width
                        height: parent.height
                        contentWidth: parent.height
                        contentHeight: parent.width
                        clip: true
                        boundsBehavior: Flickable.StopAtBounds

                        TextArea.flickable: TextArea
                        {
                            id: chat_text

                            //anchors.centerIn: chat
                            Layout.fillWidth: true
                            width: chat.width
                            height: chat.height
                            wrapMode: TextArea.Wrap

                            Component.onCompleted:
                            {
                                Gamestate.chat_text = ""
                                text = Gamestate.chat_text;
                            }



                            function updateText(givenText)
                            {
                                Gamestate.chat_text =  givenText + "\n" + Gamestate.chat_text;
                                text = Gamestate.chat_text;
                            }
                        }

                        ScrollBar.vertical: ScrollBar { }
                    }
                }
            }
        }
}
