#include "webapi.h"

Webapi::Webapi() : manager(new QNetworkAccessManager() )
{

    connect(manager, SIGNAL(finished(QNetworkReply*)),
            this, SLOT(cardDownloadFinished(QNetworkReply*)));
}

void Webapi::getUrl(const QString &_name)
{
    //https://api.scryfall.com/cards/named?fuzzy=baron+sengir
    //https://api.scryfall.com/cards/e5fb44d7-43c1-4fa2-9d55-aea6154a676d

    if(nameToImages.contains(_name))
    {
        emit(downloadDone(_name, nameToImages.value(_name).first,  nameToImages.value(_name).second));
        return;
    }

    else
    {
        QUrl url("https://api.scryfall.com/cards/named?fuzzy="+_name);

        urlToName[url] = _name;

        if(QFile(url.toEncoded().toBase64()+".cache").exists())
        {
            cardDownloadFinishedFromCache(url.toEncoded().toBase64()+".cache",_name);
        }

        else
        {
            manager->get(QNetworkRequest(url));
        }
    }
}

void Webapi::cardDownloadFinishedFromCache(const QString& _filename, const QString &_name)
{
    //qDebug() << "Using cache for " << _name;

    QFile jsonFile(_filename);
    jsonFile.open(QFile::ReadOnly);

    QJsonDocument doc(QJsonDocument::fromJson(jsonFile.readAll()));
    QJsonObject obj(doc.object());

    QJsonValue images = obj.value("image_uris"); //"art_crop"

    const QString url_small(images.toObject()["small"].toString());
    const QString url_big(images.toObject()["normal"].toString());

    const QString& name = _name;

    nameToImages[name].first = url_small;
    nameToImages[name].second = url_big;

    jsonFile.close();

    emit(downloadDone(name,url_small,url_big));
}

void Webapi::cardDownloadFinished(QNetworkReply *reply)
{

    QJsonDocument doc(QJsonDocument::fromJson(reply->readAll()));
    QJsonObject obj(doc.object());

    QJsonValue images = obj.value("image_uris"); //"art_crop"

    const QString url_small(images.toObject()["small"].toString());
    const QString url_big(images.toObject()["normal"].toString());

    /*
    *(lookup.value(reply->url()).first) = url_small;
    *(lookup.value(reply->url()).second) = url_big;

    qDebug() << *lookup.value(reply->url()).second;
    */


    QFile cache(reply->url().toEncoded().toBase64()+".cache");
    cache.open(QFile::WriteOnly);
    cache.write(doc.toJson());
    cache.close();


    const QUrl url = reply->url();
    const QString name = urlToName.value(url);

    nameToImages[name].first = url_small;
    nameToImages[name].second = url_big;

    emit(downloadDone(name,url_small,url_big));
}
